# -*- coding: utf-8 -*-
from fargo.dom import Tag, CSS


class BaseTag(Tag):
    def __init__(self, *args, **kwargs):
        super(BaseTag, self).__init__(*args, **kwargs)
        if self.__class__.__bases__[0].__name__ == 'BaseTag':
            del self.params['class']


class Html(BaseTag):
    def __init__(self, inner):
        super(Html, self).__init__('html', inner=inner)

    def __str__(self):
        return '<!DOCTYPE html>\n%s' % super(Html, self).__str__()


class Head(BaseTag):
    def __init__(self, text):
        super(Head, self).__init__('head', inner=text)


class Title(BaseTag):
    def __init__(self, text):
        super(Title, self).__init__('title', inner=text)


class Meta(BaseTag):
    def __init__(self, params):
        super(Meta, self).__init__('meta', params=params, pair=False)


class Style(BaseTag, CSS):
    def __init__(self):
        inner = CSS().css()
        super(Style, self).__init__('style', inner=inner)


class Link(BaseTag):
    def __init__(self, params):
        super(Link, self).__init__('link', params=params, pair=False)


class Script(BaseTag):
    def __init__(self, inner, params=None):
        super(Script, self).__init__('script', inner=inner, params=params)


class Body(BaseTag):
    def __init__(self, inner):
        super(Body, self).__init__('body', inner=inner)


class Div(BaseTag):
    def __init__(self, inner):
        super(Div, self).__init__('div', inner=inner)


class P(BaseTag):
    def __init__(self, text):
        super(P, self).__init__('p', inner=text)


class H1(BaseTag):
    def __init__(self, text):
        super(H1, self).__init__('h1', inner=text)


class A(BaseTag):
    def __init__(self, text, href):
        super(A, self).__init__('a', params={'href': href}, inner=text)


class Img(BaseTag):
    def __init__(self, src, alt):
        super(Img, self).__init__('img', params={'src': src, 'alt': alt}, pair=False)


class Ul(BaseTag):
    def __init__(self, text):
        super(Ul, self).__init__('ul', inner=text)


class Li(BaseTag):
    def __init__(self, text):
        super(Li, self).__init__('li', inner=text)
