# -*- coding: utf-8 -*-

def all_subclasses(cls):
    return cls.__subclasses__() + [g for s in cls.__subclasses__() for g in all_subclasses(s)]


class Min():
    min = True
    classes = {}


class Tag(object):
    def __init__(self, name, params=None, inner='', pair=True):
        if params is None:
            params = {}

        self.name = name
        self.params = params

        if not hasattr(self, 'noclass'):
            self.params['class'] = self.__class__.__name__
            if self.__class__.__name__ not in Min.classes:
                Min.classes[self.__class__.__name__] = 'c%s' % len(Min.classes)

        self.inner = ''.join([
            str(el) for el in inner
        ]) if type(inner) in [list, tuple] else inner
        self.pair = pair

    def __str__(self):
        params_rendered = ''

        if self.params:
            params_rendered = ' '
            for param in self.params.items():
                param_name, param_value = param
                if Min.min:
                    if param_name == 'class':
                        param_value = Min.classes[param_value]

                params_rendered += '%s="%s"' % (param_name, param_value)

        if self.pair:
            html = '<%(name)s%(params)s>%(inner)s</%(name)s>' % {
                'name': self.name,
                'inner': self.inner,
                'params': params_rendered,
            }
        else:
            html = '<%(name)s%(params)s />' % {
                'name': self.name,
                'params': params_rendered,
            }

        return html


class CSS(object):
    @classmethod
    def collect_subclasses(cls):
        return cls.__subclasses__() + [g for s in cls.__subclasses__() for g in all_subclasses(s)]

    def css(self):
        if 'min_classes' not in globals():
            globals()['min_classes'] = {}

        classes = [self.__class__] + [obj for obj in self.collect_subclasses()]

        for klass in classes:
            if klass.__name__ not in Min.classes:
                Min.classes[klass.__name__] = 'c%s' % len(Min.classes)

        classes_members = {
            klass: list(
                filter(
                    lambda x: not callable(getattr(klass, x)) and not x.startswith('__'),
                    dir(klass)
                )
            ) for klass in classes
        }
        filtered_classes = {k: v for k, v in classes_members.items() if v}
        rendered_classes = {
            klass.__name__: ''.join(
                ['%s:%s;' % (str(k).replace('_', '-'), getattr(klass, k)) for k in styles]
            ) for klass, styles in filtered_classes.items()
        }

        compiled_classes = ''.join(
            ['.%s{%s}' % (
                Min.classes[css_class] if Min.min else css_class, css_style
            ) for css_class, css_style in rendered_classes.items()]
        )

        return compiled_classes
